# clustering of generated nodes
import sys
import os
import requests
import json
modules_path = './'
if os.path.exists(modules_path):
    sys.path.insert(1, modules_path)

import matplotlib.pyplot as plt
import sklearn.datasets
import numpy as np
from processing.clustering.clusterer import Clusterer

# parameters for data generation
N_SAMPLES = 1000
N_FEATURES = 2
N_CENTERS = 3
STD_DEVIATION = 1.0

def show_generated_data(ax, nodes, labels):
    distinct_colors = plt.cm.rainbow(np.linspace(0, 1, N_CENTERS))
    colors = [distinct_colors[label] for label in labels]

    ax.set_title('Generated Dataset')
    ax.set_xlabel('Feature 1')
    ax.set_ylabel('Feature 2')
    ax.scatter(nodes[:,0], nodes[:,1], c=colors)

def show_clustering_result(ax, min_pts, clusters: dict):
    labels = clusters.keys()
    # flatten values in dict
    nodes = [node for subset in clusters.values() for node in subset]
    
    if -1 in labels:
        # clustering contains noise, add them in black
        distinct_colors = plt.cm.rainbow(np.linspace(0, 1, len(set(labels))-1))
        distinct_colors = np.append(distinct_colors, [[0,0,0,1]], axis=0)
    else:
        distinct_colors = plt.cm.rainbow(np.linspace(0, 1, len(set(labels))))
    colors = [distinct_colors[node['cluster_label']] for node in nodes]

    ax.set_title(f'Clustering Result with MinPts={min_pts}')
    ax.set_xlabel('Total_Demand_MWh')
    ax.set_ylabel('Customer')
    ax.scatter( [n['Total_Demand_MWh'] for n in nodes], 
                [n['Customer'] for n in nodes], 
                c=colors)

def show_clusteringSingleFeature_result(ax, min_pts, clusters: dict):
    labels = clusters.keys()
    # flatten values in dict
    nodes = [node for subset in clusters.values() for node in subset]
    
    if -1 in labels:
        # clustering contains noise, add them in black
        distinct_colors = plt.cm.rainbow(np.linspace(0, 1, len(set(labels))-1))
        distinct_colors = np.append(distinct_colors, [[0,0,0,1]], axis=0)
    else:
        distinct_colors = plt.cm.rainbow(np.linspace(0, 1, len(set(labels))))
    colors = [distinct_colors[node['cluster_label']] for node in nodes]

    ax.set_title(f'Clustering Result with MinPts={min_pts}')
    ax.set_xlabel('Total_Demand_MWh')
    ax.set_ylabel('Customer')
    ax.scatter( [n['Total_Demand_MWh'] for n in nodes], 
                [1 for n in nodes],
                c=colors)


def run_clustering(min_points, dataset):
    clusterer = Clusterer(min_points=min_points)
    return clusterer.cluster_dataset(
        dataset=dataset,
        features=['Total_Demand_MWh','Customer']
    )

def run_clustering_SingleFeature(min_points, dataset):
    clusterer = Clusterer(min_points=min_points)
    return clusterer.cluster_dataset(
        dataset=dataset,
        features=['Total_Demand_MWh']
    )

    # res: Dict[Any, ClusterResult] = clusterer.cluster_dataset(
    #     nodes,
    #     layer.properties
    # )


if __name__ == '__main__':
    fig, ((ax1, ax2), (ax3, ax4)) = plt.subplots(2, 2)
    fig.tight_layout(pad=3.0)

    nodes, labels = sklearn.datasets.make_blobs(n_samples=N_SAMPLES, n_features=N_FEATURES, centers=N_CENTERS, cluster_std=STD_DEVIATION)
    # nodes = np.multiply(nodes, .1)
    #get nodes from swagger
    #r.request (link)
    #nodes = blahb lbah

    #TODO get a list of values (total demand)
    #nodes =

    #USELESS NOW
    #show_generated_data(ax1, nodes, labels) 

    #dataset = [{'1':n[0], '2':n[1]} for n in nodes]

    JWT_TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ1c2VybmFtZSI6InJlZ3VsYXJAaXRlYy5hYXUuYXQiLCJjcmVhdGVkX2F0IjoiMjAyMC0xMC0yMCAxNDoyNDoxMi45MzI3OTAiLCJ2YWxpZF91bnRpbCI6IjIwMjAtMTAtMjEgMTQ6MjQ6MTIuOTMyNzkwIn0.qzaDauyEA4pAnw8K8ik6jTtbEOY24q159GDYbvByaJ4"
    #r = requests.get('https://articonf1.itec.aau.at:30103/api/paper/paper/layers/Demand_Layer/nodes', timeout=15)
    r = requests.get(
        
        url = 'https://articonf1.itec.aau.at:30103/api/paper/paper/layers/Demand_Layer/nodes',
        timeout=15,
        headers = {"Authorization": f"Bearer {JWT_TOKEN}"},
        verify = False # ignore ssl error)
    )
    #TODO NEED PERMISSION HOW DO I GET PERMISSION
    print("Downloaded JSON")
    inputSimListOfDict = json.loads(r.content)

    # {
    # "Customer": "13",
    # "Postcode": "2261",
    # "Timestamp": "2012-07-02 09:00:00",
    # "Total_Demand_MWh": "10513.24",
    # "UniqueID": "f5a4eb614bf3d794211970c65365aeeec7afe6750b7623e3de4d174f9ef0d6e1",
    # "layer_name": "Demand_Layer",
    # "use_case": "paper",
    # "use_case_table": "paper"
    # }
    
    dataset = []
    skippedCounter=0
    for entry in inputSimListOfDict:
        newDict = {}
        try: 
            newDict["Total_Demand_MWh"] = float(entry["Total_Demand_MWh"])
            newDict["Customer"] = float(entry["Customer"])
            dataset.append(newDict)
        except:
            skippedCounter+=1
            #print ("Warning: Skipped Bad formated Node")
        
    print("Warning: Skipped "+ str(skippedCounter) + " Badly formated nodes")

    print("Started 1st Clustering")
    clusters = run_clustering_SingleFeature(25, dataset)
    show_clusteringSingleFeature_result(ax1, 25, clusters)

    print("Started 2nd Clustering")
    clusters = run_clustering_SingleFeature(50, dataset)
    show_clusteringSingleFeature_result(ax2, 50, clusters)

    print("Started 3rd Clustering")
    clusters = run_clustering_SingleFeature(100, dataset)
    show_clusteringSingleFeature_result(ax3, 100, clusters)

    print("Started 4th Clustering")
    clusters = run_clustering_SingleFeature(300, dataset)
    show_clusteringSingleFeature_result(ax4, 300, clusters)
        
    # print("Started 1st Clustering")
    # clusters = run_clustering(5, dataset)
    # show_clustering_result(ax1, 5, clusters)
    
    # print("Started 2nd Clustering")
    # clusters = run_clustering(10, dataset)
    # show_clustering_result(ax2, 10, clusters)
    
    # print("Started 3rd Clustering")
    # clusters = run_clustering(15, dataset)
    # show_clustering_result(ax3, 15, clusters)
    
    # print("Started 4th Clustering")
    # clusters = run_clustering(25, dataset)
    # show_clustering_result(ax4, 25, clusters)

    

    plt.show()
    print("#FINISH")