import sys
import os
modules_path = '../../../modules/'
if os.path.exists(modules_path):
    sys.path.insert(1, modules_path)

import json
from db.entities import Layer, Cluster
from typing import List, Dict, Tuple, Any
from db.repository import Repository
from processing.clustering import Clusterer, ClusterResult


repo = Repository()


def run_generic_clustering():
    '''Runs the clustering for all layers found in the repository.'''
    all_layers:List[Layer] = repo.get_layers()
    #all_layers:List[Layer] = repo.get_layers_for_use_case("paper")

    for layer in all_layers:
        print(f"Clustering {layer.use_case}//{layer.use_case_table}//{layer.layer_name}.")

        if layer.properties is None or len(layer.properties) == 0:
            print("skipping")
            continue

        try:
            clusters = run_clustering_for_layer(layer)
            store_generic_clusters(clusters)
        except Exception as e:
            print(str(e))

def run_clustering_for_Paper_case():

    all_layers:List[Layer] = repo.get_layers_for_use_case("paper")

    for layer in all_layers:
        print(f"Clustering {layer.use_case}//{layer.use_case_table}//{layer.layer_name}.")

        if layer.properties is None or len(layer.properties) == 0:
            print("skipping")
            continue

        try:
            clusters = run_clustering_for_layer(layer)
            store_generic_clusters(clusters)
        except Exception as e:
            print(str(e))


def run_clustering_for_layer(layer: Layer) -> List[Cluster]:
    nodes = repo.get_layer_nodes(layer.use_case, layer.use_case_table, layer.layer_name)

    clusterer = Clusterer()
    res: Dict[Any, ClusterResult] = clusterer.cluster_dataset(
        nodes,
        layer.properties
    )

    return [Cluster(layer.use_case, layer.use_case_table, layer.layer_name, 
                    cluster_label=key, nodes=cluster_result.nodes, label=cluster_result.label if key != -1 else 'noise') 
            for key, cluster_result in res.items()]


def store_generic_clusters(clusters: List[Cluster]):
    repo.add_clusters(clusters)


if __name__ == "__main__":
    repo.delete_all_clusters()
    run_generic_clustering()
