# clustering of generated nodes
import sys
import os
import requests
import json
modules_path = './'
if os.path.exists(modules_path):
    sys.path.insert(1, modules_path)

import matplotlib.pyplot as plt
import sklearn.datasets
import numpy as np
from processing.clustering.clusterer import Clusterer
#from datascience import stats

# parameters for data generation
N_SAMPLES = 1000
N_FEATURES = 2
N_CENTERS = 3
STD_DEVIATION = 1.0

def show_generated_data(ax, nodes, labels):
    distinct_colors = plt.cm.rainbow(np.linspace(0, 1, N_CENTERS))
    colors = [distinct_colors[label] for label in labels]

    ax.set_title('Generated Dataset')
    ax.set_xlabel('Feature 1')
    ax.set_ylabel('Feature 2')
    ax.scatter(nodes[:,0], nodes[:,1], c=colors)

def show_clustering_result(ax, min_pts, clusters: dict):
    labels = clusters.keys()
    # flatten values in dict
    #nodes = [node for subset in clusters.values() for node in subset]
    # ^^^ bugged| replaced vvv
    nodes = []
    for lbl in labels:
        nodes.extend(clusters[lbl].nodes)
    
    if -1 in labels:
        # clustering contains noise, add them in black
        distinct_colors = plt.cm.rainbow(np.linspace(0, 1, len(set(labels))-1))
        distinct_colors = np.append(distinct_colors, [[0,0,0,1]], axis=0)
    else:
        distinct_colors = plt.cm.rainbow(np.linspace(0, 1, len(set(labels))))
    colors = [distinct_colors[node['cluster_label']] for node in nodes]

    ax.set_title(f'Clustering Result with MinPts={min_pts}')
    ax.set_xlabel('User_Demand_kWh')
    ax.set_ylabel('Customer')
    ax.scatter( [n['User_Demand_kWh'] for n in nodes], 
                [n['Customer'] for n in nodes], 
                c=colors)

def show_clusteringSingleFeature_result(ax, min_pts, clusters: dict):
    labels = clusters.keys()
    # flatten values in dict

    
    #nodes = [node for subset in clusters.values() for node in subset]
    # ^^^ bugged| replaced vvv
    nodes = []
    for lbl in labels:
        nodes.extend(clusters[lbl].nodes)
    
    if -1 in labels:
        # clustering contains noise, add them in black
        distinct_colors = plt.cm.rainbow(np.linspace(0, 1, len(set(labels))-1))
        distinct_colors = np.append(distinct_colors, [[0,0,0,1]], axis=0)
    else:
        distinct_colors = plt.cm.rainbow(np.linspace(0, 1, len(set(labels))))
    colors = [distinct_colors[node['cluster_label']] for node in nodes]

    ax.set_title(f'Clustering Result with MinPts={min_pts}')
    ax.set_xlabel('User_Demand_kWh')
    ax.grid(True, axis='x')
    #ax.set_ylabel('Customer')
    ax.scatter( [n['User_Demand_kWh'] for n in nodes], 
                [0 for n in nodes],
                c=colors)

def show_clusteringSingleFeatureDensityBoxPlot_result(ax, min_pts, clusters: dict):
    labels = clusters.keys()
    # flatten values in dict

    
    #nodes = [node for subset in clusters.values() for node in subset]
    # ^^^ bugged| replaced vvv
    nodes = []
    mydict = dict()
    for lbl in labels:
        nodes.extend(clusters[lbl].nodes)
    
    for n in nodes: #group nodes per clusters
        if not checkKey(mydict,n['cluster_label']):
            mydict[n['cluster_label']] = []

        mydict[n['cluster_label']].append(n['User_Demand_kWh'])

    

    
    
    
    if -1 in labels:
        # clustering contains noise, add them in black
        distinct_colors = plt.cm.rainbow(np.linspace(0, 1, len(set(labels))-1))
        distinct_colors = np.append(distinct_colors, [[0,0,0,1]], axis=0)
    else:
        distinct_colors = plt.cm.rainbow(np.linspace(0, 1, len(set(labels))))
    colors = [distinct_colors[node['cluster_label']] for node in nodes]

    for cLabel, cValue in mydict.items():
        cluster_label = int(cLabel)
        if cluster_label != -1:
            bp = ax.boxplot(cValue, positions = [0],vert=False, widths = 0.15)
            plt.setp(bp['boxes'][0], color=distinct_colors[cluster_label])
            plt.setp(bp['caps'][0], color=distinct_colors[cluster_label])
            plt.setp(bp['caps'][1], color=distinct_colors[cluster_label])
            plt.setp(bp['whiskers'][0], color=distinct_colors[cluster_label])
            plt.setp(bp['whiskers'][1], color=distinct_colors[cluster_label])
            try:
                plt.setp(bp['fliers'][0], color=distinct_colors[cluster_label])
            except:
                print('')
            try: 
                plt.setp(bp['fliers'][1], color=distinct_colors[cluster_label])
            except:
                print('')
            plt.setp(bp['medians'][0], color=distinct_colors[cluster_label])
    
    ax.set_title(f'Clustering Result with MinPts={min_pts}')
    ax.set_xlabel('User_Demand_kWh')
    #ax.set_ylabel('Cluster Label')
    ax.grid(True, axis='x')
    # ax.scatter( [n['User_Demand_kWh'] for n in nodes], 
    #             [0 for n in nodes],
    #             c=colors)



def run_clustering(min_points, dataset):
    clusterer = Clusterer(min_points=min_points)
    return clusterer.cluster_dataset(
        dataset=dataset,
        features=['User_Demand_kWh','Customer']
    )

def run_clustering_SingleFeature(min_points, dataset):
    clusterer = Clusterer(min_points=min_points)
    return clusterer.cluster_dataset(
        dataset=dataset,
        features=['User_Demand_kWh']
    )

    # res: Dict[Any, ClusterResult] = clusterer.cluster_dataset(
    #     nodes,
    #     layer.properties
    # )

def checkKey(dict, key):       
    if key in dict.keys(): 
        #print("Present, ", end =" ") 
        #print(str(key)+ " : " + str(dict[key] ))
        return True
    else: 
        #print("Not present") 
        return False
    
def createDataset(inputDict):
    dataset = []
    skippedCounter=0
    for entry in inputSimListOfDict:
        newDict = {}
        try: 
            newDict["User_Demand_kWh"] = float(entry["User_Demand_kWh"])
            newDict["Customer"] = float(entry["Customer"])
            dataset.append(newDict)
        except:
            skippedCounter+=1
            #print ("Warning: Skipped Bad formated Node")
    
    print("Warning: Skipped "+ str(skippedCounter) + " Badly formated nodes")
    return dataset  



if __name__ == '__main__':
    fig, ((ax1, ax2), (ax3, ax4)) = plt.subplots(2, 2)
    fig.tight_layout(pad=3.0)
    
    nodes, labels = sklearn.datasets.make_blobs(n_samples=N_SAMPLES, n_features=N_FEATURES, centers=N_CENTERS, cluster_std=STD_DEVIATION)
    # nodes = np.multiply(nodes, .1)
    #get nodes from swagger
    #r.request (link)
    #nodes = blahb lbah

    #TODO get a list of values (total demand)
    #nodes =

    #USELESS NOW
    #show_generated_data(ax1, nodes, labels) 

    #dataset = [{'1':n[0], '2':n[1]} for n in nodes]

    JWT_TOKEN = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ1c2VybmFtZSI6InJlZ3VsYXJAaXRlYy5hYXUuYXQiLCJjcmVhdGVkX2F0IjoiMjAyMC0xMC0yMCAxNDoyNDoxMi45MzI3OTAiLCJ2YWxpZF91bnRpbCI6IjIwMjAtMTAtMjEgMTQ6MjQ6MTIuOTMyNzkwIn0.qzaDauyEA4pAnw8K8ik6jTtbEOY24q159GDYbvByaJ4"

    r = requests.get(
        
        url = 'https://articonf1.itec.aau.at:30103/api/paper/paper/layers/User_Demand_Layer/nodes',
        timeout=15,
        headers = {"Authorization": f"Bearer {JWT_TOKEN}"},
        verify = False # ignore ssl error)
    )
    #TODO NEED PERMISSION HOW DO I GET PERMISSION
    print("Downloaded JSON")
    inputSimListOfDict = json.loads(r.content)

    # {
    # "Customer": "13",
    # "Postcode": "2261",
    # "Timestamp": "2012-07-02 09:00:00",
    # "Total_Demand_MWh": "10513.24",
    # "UniqueID": "f5a4eb614bf3d794211970c65365aeeec7afe6750b7623e3de4d174f9ef0d6e1",
    # "layer_name": "Demand_Layer",
    # "use_case": "paper",
    # "use_case_table": "paper"
    # }
    
    # dataset = []
    # skippedCounter=0
    # for entry in inputSimListOfDict:
    #     newDict = {}
    #     try: 
    #         newDict["Total_Demand_MWh"] = float(entry["Total_Demand_MWh"])
    #         newDict["Customer"] = float(entry["Customer"])
    #         dataset.append(newDict)
    #     except:
    #         skippedCounter+=1
    #         #print ("Warning: Skipped Bad formated Node")
        
    
    # dataset = createDataset(inputSimListOfDict)
    # print("Started TEST Clustering") #500 = 4 clusters
    # clusters = run_clustering_SingleFeature(250, dataset)
    # show_clusteringSingleFeatureDensityBoxPlot_result(ax1,250,clusters)
   
    # dataset = createDataset(inputSimListOfDict)
    # print("Started TEST Clustering") #500 = 4 clusters
    # clusters = run_clustering_SingleFeature(500, dataset)
    # show_clusteringSingleFeatureDensityBoxPlot_result(ax2,500,clusters)

    # dataset = createDataset(inputSimListOfDict)
    # print("Started TEST Clustering") #500 = 4 clusters
    # clusters = run_clustering_SingleFeature(750, dataset)
    # show_clusteringSingleFeatureDensityBoxPlot_result(ax3,750,clusters)

    # dataset = createDataset(inputSimListOfDict)
    # print("Started TEST Clustering") #500 = 4 clusters
    # clusters = run_clustering_SingleFeature(1000, dataset)
    # show_clusteringSingleFeatureDensityBoxPlot_result(ax4,1000,clusters)

    

    # print("Started 1st Clustering")
    # dataset = createDataset(inputSimListOfDict)
    # clusters = run_clustering_SingleFeature(15, dataset)
    # show_clusteringSingleFeature_result(ax1, 15, clusters)

    # print("Started 2nd Clustering")
    # dataset = createDataset(inputSimListOfDict)
    # clusters = run_clustering_SingleFeature(25, dataset)
    # show_clusteringSingleFeature_result(ax2, 25, clusters)

    
    # print("Started 3rd Clustering")
    # dataset = createDataset(inputSimListOfDict)
    # clusters = run_clustering_SingleFeature(50, dataset)
    # show_clusteringSingleFeature_result(ax3, 50, clusters)

    # print("Started 4th Clustering")
    # dataset = createDataset(inputSimListOfDict)
    # clusters = run_clustering_SingleFeature(100, dataset)
    # show_clusteringSingleFeature_result(ax4, 100, clusters)

    
    dataset = createDataset(inputSimListOfDict)    
    print("Started 1st Clustering")
    clusters = run_clustering(10, dataset)
    show_clustering_result(ax1, 10, clusters)
    
    dataset = createDataset(inputSimListOfDict)
    print("Started 2nd Clustering")
    clusters = run_clustering(15, dataset)
    show_clustering_result(ax2, 15, clusters)
    
    dataset = createDataset(inputSimListOfDict)
    print("Started 3rd Clustering")
    clusters = run_clustering(25, dataset)
    show_clustering_result(ax3, 25, clusters)

    dataset = createDataset(inputSimListOfDict)
    print("Started 4th Clustering")
    clusters = run_clustering(50, dataset)
    show_clustering_result(ax4, 50, clusters)

    #agePhysics = [ 25, 31, 31, 31, 12,28,29,31,33,34,35,36,34,39,40,41,48 ]
    # basic plot
    #plt.boxplot(agePhysics)
    #plt.boxplot(agePhysics, showmeans=True)
    

    plt.show()
    print("#FINISH")