import csv
import hashlib
import datetime

import sys
import os
modules_path = '../../../modules/'
if os.path.exists(modules_path):
    sys.path.insert(1, modules_path)

from messaging.MessageHandler import MessageHandler

# file to read the data from
CSV_FILE = r'Energy_Dataset.csv'
handler = MessageHandler()

from db.repository import *
repo = Repository()


def upload_transaction(transaction):
    '''{"type": "new-trace", 
    "content": {"use_case": "paper", "table": "paper", "id": "dd2c5146c919b046d77a32a5cf553d5133163562f7b7e1298c878c575d516025", 
    "properties": {"Customer": "297", "Energy_Consumption_kWh": "0.177", "Heating_Consumption_kWh": "0.0", "Latitude": "-33.362679", "Longitude": "151.447302", "Postcode": "2261", "Price_AUD_MWh": "58.05", "Solar_Production_kWh": "0.0", "Timestamp": "2013-06-30 00:00:00", "Total_Demand_MWh": "8154.14", "UniqueID": "dd2c5146c919b046d77a32a5cf553d5133163562f7b7e1298c878c575d516025"}
    }}'''
    uid = hashlib.sha256(f"{transaction['Customer']}{transaction['Timestamp']}".encode("utf-8")).hexdigest()
    transaction['UniqueID'] = uid
    t = {        
        'use_case': 'paper',
        'table': 'paper',
        'id': uid,
        'properties': transaction
    }
    handler.handle_new_trace(t)


if __name__ == '__main__':    

    repo.delete_nodes_for_use_case("paper")
    print ("###DELETED NODES FROM SEMANTIC LINKING - PAPER USE CASE")
    transactionList = []
    with open(CSV_FILE, 'r') as file:
        reader = csv.reader(file)

        titles = next(reader)

        #################
        maxCustomers = 10
        maxRowsPerMonth = 120 # 24Hr * 5Days
        month_memory=[0,0,0,0,0,0,0,0,0,0,0,0]
        ##################

        old_c = None
        customerCount = 0
        rowCount = 0
        for row in reader:
            
            transaction = {}
            transaction['ApplicationType'] = 'paper'
            transaction['docType'] = 'paper'
            for idx in range(len(row)):
                transaction[titles[idx]] = row[idx]
                
            # also include the user demand, as Total_Demand_WMh is not per user
            energy_cons = float(transaction['Energy_Consumption_kWh']) if transaction['Energy_Consumption_kWh'] is not None and transaction['Energy_Consumption_kWh'] != "" else 0
            heating_cons = float(transaction['Heating_Consumption_kWh']) if transaction['Heating_Consumption_kWh'] is not None and transaction['Heating_Consumption_kWh'] != "" else 0
            transaction['User_Demand_kWh'] = heating_cons + energy_cons

            if transaction['Customer'] != old_c:  
                customerCount +=1    
                rowCount = 0
                for i in range(12):
                    month_memory[i]=maxRowsPerMonth
                old_c = transaction['Customer']
                
            if (customerCount>maxCustomers):
                print("\nMAX customers reached")
                break
            upload_condition = False

            datetime_obj= datetime.datetime.strptime(transaction['Timestamp'],'%Y-%m-%d %H:%M:%S')
            
            if(month_memory[datetime_obj.month-1]>0): #MAX  XX times per month considered per customer
                if(datetime_obj.minute==0):             #upload only hourly
                    month_memory[datetime_obj.month-1]-=1
                    upload_condition = True

            #datetime_obj.month
            
            if(upload_condition):
                upload_transaction(transaction)
                transactionList.append(transaction)
                #print(f"uploading for {old_c}")
            rowCount+=1


    outputJSON = json.dumps(transactionList, default=lambda o: o.__dict__, indent=4)

    try: 
        with open('EnergyDataSubset.json', 'w') as outfile:
            outfile.write(outputJSON)
    except ValueError:
        print("Error occured when writing the resultSimilarityDict file")

